/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

static Widget text, dialog, label;

/* Callback Function called when "File" Menu is selected. */
static void FileMenuCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    int menu_item = (int)client_data;  /* Selected menu item number */
    printf("File Menu: item %d is selected\n", menu_item);

    if (menu_item == 0)  /* If the selected item is 0 ("New"), */
        XmTextSetString(text, "");  /* clear the text. */

    if (menu_item == 1)  /* If the selected item is 1 ("Open"), */
        XtManageChild(dialog);  /* show the file selection dialog. */

    if (menu_item == 3)  /* If the selected item is 3 ("Exit"), */
        exit(0);         /* exit this program. */
}

/* Callback function when dialog "OK" button is selected. */
static void OpenFileCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    char *filename;
    FILE *fp;
    char buff[BUFSIZ];
    XmFileSelectionBoxCallbackStruct *cb =
                (XmFileSelectionBoxCallbackStruct *)call_data;

    /* Get the string of entered File Name */
    XmStringGetLtoR(cb->value, XmFONTLIST_DEFAULT_TAG, &filename);

    /* Open the file and show the content into the text */
    XmTextSetString(text, "");  /* Once clear the text */
    fp = fopen(filename, "r");  /* Open the file */
    while (fgets(buff, BUFSIZ, fp) != NULL) {  /* Read from the file */
        /* Append the read data to the text */
        XmTextInsert(text, XmTextGetLastPosition(text), buff);
    }
    fclose(fp);
    XtFree(filename);

    /* Show the file name on the bottom label */
    XtVaSetValues(label, XmNlabelString, cb->value, NULL);
    XtUnmanageChild(dialog); /* Close the dialog */
}

/* Create menu bar and pulldown menu */
static Widget CreateMenu(parent)
    Widget parent;
{
    Widget  menubar, filemenu;

    /* Create the Menu Bar */
    menubar = XmVaCreateSimpleMenuBar(parent, "menubar",
                  XmVaCASCADEBUTTON, NULL, NULL,  /* "File" menu */
                  NULL);
    XtManageChild(menubar);

    /* Create the "File" pulldown menu */
    filemenu = XmVaCreateSimplePulldownMenu(menubar, "filemenu", 0, FileMenuCB,
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "New" */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "Open" */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "Save" */
                  XmVaSEPARATOR,  /* Separator of the menu items */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "Exit" */
                  NULL);

    return(menubar);
}

/* Copy the selected text to clipboard */
static void CopyCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    XmTextCopy(text, XtLastTimestampProcessed(XtDisplay(text)));
}

/* Cut the selected text to clipboard */
static void CutCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    XmTextCut(text, XtLastTimestampProcessed(XtDisplay(text)));
}

/* Paste the data in clipboard into the text */
static void PasteCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    XmTextPaste(text);
}

/* Create Command Area */
static Widget CreateCommandArea(parent)
    Widget parent;
{
    Widget command, copy, cut, paste;
    Arg    al[20];
    int    ac;

    ac = 0;
    command = XmCreateRowColumn(parent, "command", al, ac);
    XtManageChild(command);

    /* Create "Copy", "Cut", and "Paste" button */
    ac = 0;
    copy = XmCreatePushButton(command, "copy", al, ac);
    XtManageChild(copy);
    ac = 0;
    cut = XmCreatePushButton(command, "cut", al, ac);
    XtManageChild(cut);
    ac = 0;
    paste = XmCreatePushButton(command, "paste", al, ac);
    XtManageChild(paste);

    XtAddCallback(copy,  XmNactivateCallback, CopyCB,  NULL);
    XtAddCallback(cut,   XmNactivateCallback, CutCB,   NULL);
    XtAddCallback(paste, XmNactivateCallback, PasteCB, NULL);

    return command;
}

/* Create Message Area */
static Widget CreateMessageArea(parent)
    Widget parent;
{
    Widget message, frame, ftitle, form, ltitle;
    Arg    al[20];
    int    ac;

    ac = 0;
    XtSetArg(al[ac], XmNmarginWidth,  8); ac++;
    XtSetArg(al[ac], XmNmarginHeight, 8); ac++;
    message = XmCreateRowColumn(parent, "message", al, ac);
    XtManageChild(message);

    /* Enclose by Frame */
    ac = 0;
    XtSetArg(al[ac], XmNmarginWidth,  8); ac++;
    XtSetArg(al[ac], XmNmarginHeight, 8); ac++;
    frame = XmCreateFrame(message, "frame", al, ac);
    XtManageChild(frame);

    /* Attach a title on the frame */
    ac = 0;
    XtSetArg(al[ac], XmNchildType,                XmFRAME_TITLE_CHILD); ac++;
    XtSetArg(al[ac], XmNchildHorizontalAlignment, XmALIGNMENT_CENTER); ac++;
    XtSetArg(al[ac], XmNmarginWidth,              10); ac++;
    ftitle = XmCreateLabel(frame, "ftitle", al, ac);
    XtManageChild(ftitle);

    /* Layout the children by Form */
    ac = 0;
    form = XmCreateForm(frame, "form", al, ac);
    XtManageChild(form);

    /* Locate the label "File Name:" at the left of the center */
    ac = 0;
    XtSetArg(al[ac], XmNtopAttachment,    XmATTACH_FORM); ac++;
    XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
    XtSetArg(al[ac], XmNleftAttachment,   XmATTACH_POSITION); ac++;
    XtSetArg(al[ac], XmNleftPosition,     0); ac++;
    XtSetArg(al[ac], XmNrightAttachment,  XmATTACH_POSITION); ac++;
    XtSetArg(al[ac], XmNrightPosition,    50); ac++;
    XtSetArg(al[ac], XmNrightOffset,      4); ac++;
    XtSetArg(al[ac], XmNalignment,        XmALIGNMENT_END); ac++;
    XtSetArg(al[ac], XmNresizable,        False); ac++;
    ltitle = XmCreateLabel(form, "ltitle", al, ac);
    XtManageChild(ltitle);

    /* Locate the file name at the right of the center */
    ac = 0;
    XtSetArg(al[ac], XmNtopAttachment,    XmATTACH_FORM); ac++;
    XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
    XtSetArg(al[ac], XmNleftAttachment,   XmATTACH_POSITION); ac++;
    XtSetArg(al[ac], XmNleftPosition,     50); ac++;
    XtSetArg(al[ac], XmNleftOffset,       4); ac++;
    XtSetArg(al[ac], XmNrightAttachment,  XmATTACH_POSITION); ac++;
    XtSetArg(al[ac], XmNrightPosition,    100); ac++;
    XtSetArg(al[ac], XmNalignment,        XmALIGNMENT_BEGINNING); ac++;
    XtSetArg(al[ac], XmNresizable,        False); ac++;
    label = XmCreateLabel(form, "label", al, ac);
    XtManageChild(label);

    return message;
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, mainwin, menubar, work, command, message;
    Arg    al[20];
    int    ac;

    XtSetLanguageProc(NULL, NULL, NULL);

    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Editor", NULL, 0,
                               &argc, argv, NULL, al, ac);

    ac = 0;
        /* Show separators which span the main window horizontally. */
    XtSetArg(al[ac], XmNshowSeparator, True); ac++;
        /* Place the Command Area below Work Area */
    XtSetArg(al[ac], XmNcommandWindowLocation, XmCOMMAND_BELOW_WORKSPACE); ac++;
        /* Create Main Window */
    mainwin = XmCreateMainWindow(toplevel, "mainwin", al, ac);
    XtManageChild(mainwin);

    menubar = CreateMenu(mainwin); /* Create Menu */

        /* Work Area */
    ac = 0;
    work = XmCreateRowColumn(mainwin, "work", al, ac);
    XtManageChild(work);
    ac = 0;
    text = XmCreateText(work, "text", al, ac);
    XtManageChild(text);

        /* Command Area */
    command = CreateCommandArea(mainwin);
        /* Message Area */
    message = CreateMessageArea(mainwin);

        /* Locate each area on the Main Window */
    XmMainWindowSetAreas(mainwin, menubar, command, NULL, NULL, work);
        /* Set the Message area by resource */
    XtVaSetValues(mainwin, XmNmessageWindow, message, NULL);

        /* Create file selection dialog */
    ac = 0;
    dialog = XmCreateFileSelectionDialog(toplevel, "dialog", al, ac);
    XtAddCallback(dialog, XmNokCallback, OpenFileCB, NULL);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}
