/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>
#include <X11/cursorfont.h>  /* Necessary for creating cursor */

static Boolean modal = False; /* Now is prohibition mode or not */
static Cursor prohibit, arrow; /* Cursors used on prohibition mode */
static Widget toplevel;

/* (1) Calback for "Prohibit" button */
static void StartCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget start = (Widget)w;
    Widget done  = (Widget)client_data;
    XEvent event;

    modal = True;  /* Now in prohibition mode */
    XtSetSensitive(done,  True);  /* enable "Restore" button */
                                  /* set focus to "Restore" button */
    XmProcessTraversal(done, XmTRAVERSE_CURRENT);

    /* (2) Set the prohibition cursor */
    XDefineCursor(XtDisplay(toplevel), XtWindow(toplevel), prohibit);
    /*     Remain as normal cursor on "Restore" button */
    XDefineCursor(XtDisplay(done), XtWindow(done), arrow);

    /* (3) Processing in prohibition mode -- continued until "Restore" is pressed */
    /*     Ignore events of prohibition */
    while (modal == True) {
        /* retrieve an event */
        XtAppNextEvent(XtWidgetToApplicationContext(w), &event);
        if ( (event.type == ButtonPress &&            /* If button is pressed */
                event.xany.window != XtWindow(done)) || /* check window ID */
             (event.type == KeyPress &&              /* If key is pressed */
                XmGetFocusWidget(done) != done) ) {  /* check the focus */
            XBell(XtDisplay(done), 0); /* ring the bell for prohibitted operations */
            continue;
        }
        XtDispatchEvent(&event); /* Process other (allowed) events */
    }

    /* end of prohibition -- restore normal cursor and disable "Restore" button */
    XUndefineCursor(XtDisplay(toplevel), XtWindow(toplevel));
    XUndefineCursor(XtDisplay(done), XtWindow(done));
    XtSetSensitive(done,  False);
}

/* (4) Callback for "Restore" button -- Exit from prohibition mode */
static void DoneCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    modal = False;  /* End of prohibition mode */
}

/* Callback for "Other" button */
static void OtherCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    printf("Normal X Event is processed.\n");
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget panel, start, done, other;
    Arg    al[20];
    int    ac;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Ignoreevent", NULL, 0,
                               &argc, argv, NULL, al, ac);
    ac = 0;
    panel = XmCreateRowColumn(toplevel, "panel", al, ac);
    XtManageChild(panel);

    ac = 0;
    start = XmCreatePushButton(panel, "start", al, ac);
    XtManageChild(start);

    ac = 0;
    XtSetArg(al[ac], XmNsensitive, False); ac++;
    done = XmCreatePushButton(panel, "done", al, ac);
    XtManageChild(done);

    ac = 0;
    other = XmCreatePushButton(panel, "other", al, ac);
    XtManageChild(other);

    XtAddCallback(start, XmNactivateCallback, StartCB, done);
    XtAddCallback(done,  XmNactivateCallback, DoneCB,  NULL);
    XtAddCallback(other, XmNactivateCallback, OtherCB, NULL);

    XtRealizeWidget(toplevel);

    /* Create cursors for prohibition mode */
    prohibit = XCreateFontCursor(XtDisplay(toplevel), XC_pirate);
    arrow = XCreateFontCursor(XtDisplay(toplevel), XC_left_ptr);

    XtAppMainLoop(app_context);
}
