/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

static Boolean finished = True;  /* A flag indicating whether a condition is met or not */

/* Called when "Start Second Loop" button is pressed */
static void StartCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget start = (Widget)w;
    Widget done  = (Widget)client_data;
    XEvent event;

    finished = False;
    XtSetSensitive(start, False);  /* Disable "Start Second Loop" button */
    XtSetSensitive(done,  True);   /* Enable "Stop Second Loop" button */

    /* Loop the second event loop until some condition met */
    while (finished != True) {
        /* Retrieve and process one event */
        XtAppNextEvent(XtWidgetToApplicationContext(w), &event);
        XtDispatchEvent(&event);
        printf("Second event loop\n");
    }

    XtSetSensitive(start, True);   /* Enable "Start Second Loop" button */
    XtSetSensitive(done,  False);  /* Disable "Stop Second Loop" button */
}

/* Called when "Stop Second Loop" button is pressed */
static void DoneCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    finished = True;  /* The condition is now met */
}

/* Called when "Perform Other Operation" button is pressed */
static void OtherCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    printf("Normal X Event is processed.\n");  /* Perform other task */
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, panel, start, done, other;
    Arg    al[20];
    int    ac;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Secondloop", NULL, 0,
                               &argc, argv, NULL, al, ac);
    ac = 0;
    panel = XmCreateRowColumn(toplevel, "panel", al, ac);
    XtManageChild(panel);

    ac = 0;
    start = XmCreatePushButton(panel, "start", al, ac);
    XtManageChild(start);

    ac = 0;
    XtSetArg(al[ac], XmNsensitive, False); ac++;
    done = XmCreatePushButton(panel, "done", al, ac);
    XtManageChild(done);

    ac = 0;
    other = XmCreatePushButton(panel, "other", al, ac);
    XtManageChild(other);

    XtAddCallback(start, XmNactivateCallback, StartCB, done);
    XtAddCallback(done,  XmNactivateCallback, DoneCB, NULL);
    XtAddCallback(other, XmNactivateCallback, OtherCB, NULL);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}
