/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

#define FRAGMENT    2    /* The amount of one fragment (%) */
static Boolean WorkProc();
static Widget dialog;
static XtWorkProcId id;
static int progress;     /* Progress:  0 = start,  100 = finished */

/* Show the progress % on the label */
static void ShowProgress(w, percent)
    Widget w;        /* Widget ID of the label */
    int    percent;  /* % of the progress */
{
    char     string[BUFSIZ];
    XmString labelstr;

    sprintf(string, "%d %%", percent);  /* Create the string */
    labelstr = XmStringCreateLocalized(string); /* compound string */
    XtVaSetValues(w, XmNmessageString, labelstr, NULL);  /* show on the label */
    XmStringFree(labelstr);  /* It's not necessary any more */
}

/* Called when "Start Lengthy Task" button is pressed */
static void StartCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    progress = 0;  /* reset the progress */
    ShowProgress(dialog, progress);  /* show as "0 %" */
    XtManageChild(dialog);  /* show working dialog */
    /* Register work procedure */
    id = XtAppAddWorkProc(XtWidgetToApplicationContext(w), WorkProc, NULL);
}

/* Work procedure (called when no event is qued) */
static Boolean WorkProc(client_data)
    XtPointer client_data;  /* Client data passed at the registration */
{
    int i, j;
    for (i = 0; i < 5000000; i++)  /* perform one fragment of lengty task */
        j = i * i;  /* replace this line to the actual task */

    progress += FRAGMENT;  /* update the % of progress */
    ShowProgress(dialog, progress);  /* show the progress on the label */

    if (progress >= 100) {        /* If the task is finished, */
        XtUnmanageChild(dialog);  /* close the working dialog and */
        return True;    /* return True so that this func is no more called */
    }
    else
        return False;   /* The task is not finished.  Call this func again */
}

/* Called when "Stop" button is pressed */
static void CancelCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    XtRemoveWorkProc(id);  /* Delete the work procedure */
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget toplevel, start;
    Arg    al[20];
    int    ac;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Workproc", NULL, 0,
                               &argc, argv, NULL, al, ac);
    /* Create working dialog */
    ac = 0;                          /* set dialog mode */
    XtSetArg(al[ac], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); ac++;
    dialog = XmCreateWorkingDialog(toplevel, "dialog", al, ac);
    /* "OK" and "Help" button are not necessary */
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
    /* Register a callback for "Stop" button */
    XtAddCallback(dialog, XmNcancelCallback, CancelCB, NULL);

    ac = 0;
    start = XmCreatePushButton(toplevel, "start", al, ac);
    XtManageChild(start);
    XtAddCallback(start, XmNactivateCallback, StartCB, NULL);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}
